/* ###################################################################
**     Filename    : Events.c
**     Project     : ProcessorExpert
**     Processor   : MKL16Z128VFT4
**     Component   : Events
**     Version     : Driver 01.00
**     Compiler    : GNU C Compiler
**     Date/Time   : 2014-03-20, 11:48, # CodeGen: 0
**     Abstract    :
**         This is user's event module.
**         Put your event handler code here.
**     Settings    :
**     Contents    :
**         Cpu_OnNMIINT - void Cpu_OnNMIINT(void);
**
** ###################################################################*/
/*!
** @file Events.c
** @version 01.00
** @brief
**         This is user's event module.
**         Put your event handler code here.
*/         
/*!
**  @addtogroup Events_module Events module documentation
**  @{
*/         
/* MODULE Events */

/* Including needed modules to compile this module/procedure */
#include "Cpu.h"
#include "Events.h"
#include "UART0.h"
/* Including shared modules, which are used for whole project */
#include "PE_Types.h"
#include "PE_Error.h"
#include "PE_Const.h"
#include "IO_Map.h"

#include "WaRP_DB.h"
#include "uart.h"

#include "common/common.h"

// FSL Power Receiver Includes
#include "derivative.h"
#include "prx_lib.h"

#ifdef __cplusplus
extern "C" {
#endif 


/* User includes (#include below this line is not maintained by Processor Expert) */

/*
** ===================================================================
**     Event       :  Cpu_OnNMIINT (module Events)
**
**     Component   :  Cpu [MKL16Z256LH4]
*/
/*!
**     @brief
**         This event is called when the Non maskable interrupt had
**         occurred. This event is automatically enabled when the [NMI
**         interrupt] property is set to 'Enabled'.
*/
/* ===================================================================*/
void Cpu_OnNMIINT(void)
{
  /* Write your code here ... */
}

/*
** ===================================================================
**     Event       :  Cpu_OnReset (module Events)
**
**     Component   :  Cpu [MKL16Z128FT4]
*/
/*!
**     @brief
**         This software event is called after a reset.
**     @param
**         Reason          - Content of the reset status register.
**                           You can use predefined constants RSTSRC_*
**                           defined in generated PE_Const.h file to
**                           determine a reason of the last reset. See
**                           definition of these constants in this file
**                           for details.
*/
/* ===================================================================*/
void Cpu_OnReset(uint16_t Reason)
{
  /* Write your code here ... */
}

/*
** ===================================================================
**     Event       :  Cpu_OnLLSWakeUpINT (module Events)
**
**     Component   :  Cpu [MKL16Z128FT4]
*/
/*!
**     @brief
**         This event is called when Low Leakage WakeUp interrupt
**         occurs. LLWU flags indicating source of the wakeup can be
**         obtained by calling the [GetLLSWakeUpFlags] method. Flags
**         indicating the external pin wakeup source are automatically
**         cleared after this event is executed. It is responsibility
**         of user to clear flags corresponding to internal modules.
**         This event is automatically enabled when [LLWU interrupt
**         request] is enabled.
*/
/* ===================================================================*/
void Cpu_OnLLSWakeUpINT(void)
{
  /* Write your code here ... */
}


void PORTC_PORTD_ISR(void)
{
	static char currBTN0 = 2;
	static char prevBTN0 = 2;
	static char currBTN1 = 2;
	static char prevBTN1 = 2;
	
	if(PORTC_ISFR){
		if(PORTC_ISFR & 0x8){
			PORTC_ISFR |= 0x8;
			// Handle Button 0 press here
			currBTN0 = ((GPIOC_PDIR & (0x8)) >> 3);
			if(!debounceCounter && (currBTN0 != prevBTN0)){
				if(currBTN0){
					eventQueuePut(&EventQueue, BTN0_UP);
				}else{
					eventQueuePut(&EventQueue, BTN0_DOWN);
				}
				prevBTN0 = currBTN0;
				debounceCounter = DEBOUNCE_CYCLES;
			}
		}else if (PORTC_ISFR & 0x10){
			PORTC_ISFR |= 0x10;
			// Handle Button 0 press here
			currBTN1 = ((GPIOC_PDIR & (0x10)) >> 4);
			if(!debounceCounter && (currBTN1 != prevBTN1)){
				if(currBTN1){
					eventQueuePut(&EventQueue, BTN1_UP);
				}else{
					eventQueuePut(&EventQueue, BTN1_DOWN);
				}
				prevBTN1 = currBTN1;
				debounceCounter = DEBOUNCE_CYCLES;
			}
		}	
	}
}
void UART0_ISR(void){
	char ch;
	// Check if TX data is available and can be sent
	if(uartTxRegEmpty() && UartTxQueue.count > 0){
		uartQueueGet(&UartTxQueue, &ch);
		UART0_D = ch;
	}
	// If no data is left in the TX queue, disable the transmit interrupt,
	// it will be reenabled each time data is added to the queue.
	if(UartTxQueue.count == 0){
		disableUart0TxInt();
	}
	// Check if RX data is available and move it to the receive queue
	if(uartRxRegFull()){
		ch = UART0_D;
		uartQueuePut(&UartRxQueue, ch);
	}
}


/*
** ===================================================================
**     Event       :  SM1_OnBlockSent (module Events)
**
**     Component   :  SM1 [SPIMaster_LDD]
*/
/*!
**     @brief
**         This event is called after the last character from the
**         output buffer is moved to the transmitter. This event is
**         available only if the SendBlock method is enabled.
**     @param
**         UserDataPtr     - Pointer to the user or
**                           RTOS specific data. The pointer is passed
**                           as the parameter of Init method. 
*/
/* ===================================================================*/
void SM1_OnBlockSent(LDD_TUserData *UserDataPtr)
{
  /* Write your code here ... */
}

/*
** ===================================================================
**     Event       :  SM1_OnBlockReceived (module Events)
**
**     Component   :  SM1 [SPIMaster_LDD]
*/
/*!
**     @brief
**         This event is called when the requested number of data is
**         moved to the input buffer. This method is available only if
**         the ReceiveBlock method is enabled.
**     @param
**         UserDataPtr     - Pointer to the user or
**                           RTOS specific data. The pointer is passed
**                           as the parameter of Init method. 
*/
/* ===================================================================*/
void SM1_OnBlockReceived(LDD_TUserData *UserDataPtr)
{
  /* Write your code here ... */
}

/*
** ===================================================================
**     Event       :  TI2_OnInterrupt (module Events)
**
**     Component   :  TI2 [TimerInt]
**     Description :
**         When a timer interrupt occurs this event is called (only
**         when the component is enabled - <Enable> and the events are
**         enabled - <EnableEvent>). This event is enabled only if a
**         <interrupt service/event> is enabled.
**     Parameters  : None
**     Returns     : Nothing
** ===================================================================
*/
void TI2_OnInterrupt(void)
{
  /* Write your code here ... */
	/* Send packet ! */
	packet_send();
}

/*
** ===================================================================
**     Event       :  TI1_OnInterrupt (module Events)
**
**     Component   :  TI1 [TimerInt]
**     Description :
**         When a timer interrupt occurs this event is called (only
**         when the component is enabled - <Enable> and the events are
**         enabled - <EnableEvent>). This event is enabled only if a
**         <interrupt service/event> is enabled.
**     Parameters  : None
**     Returns     : Nothing
** ===================================================================
*/
void TI1_OnInterrupt(void)
{
  /* Write your code here ... */
	/*
		 * ========================
		 * PEx Flow upon interrupt 
		 * ========================
		 * 
		 * Cpu.c->PE_ISR()
		 * 	->TU1_Interrupt()
		 * 		->TU1_OnCounterRestart()
		 * 			->TimerIntLdd1_OnInterrupt()
		 * 				->TI1_OnInterrupt()  ** here **
		 */
		
		/*
		 * This is from MTIM_ISR() 
		 */
		VUINT8 phase;
		
		/* This is a workaround that may be implemented differently in the future. 
		 * LDVAL1 gets changed during comms (from 1ms to 250us interval) but we have one ISR for both PIT channels
		 * Here we are assuming that LDVAL0 doesn't change from the 1ms interval at all, only LDVAL1 changes, 
		 * so we can simply return here if they are different because we don't want the ms ticker to increment during the 250us comms interrupt
		 * LDVAL0 is used in TI1, LDVAL1 is used for TI2 
		 */
		if ((PIT_LDVAL1 & ~0xFF) != (PIT_LDVAL0 & ~0xFF))	//ignore the lower byte.  example, 0x5dbf is used in init for 1ms, but in calculated in code it comes out to be 0x5dc0	
		{
			return;	
		}
		
		//clrReg8Bits(MTIM1SC, MTIM1SC_TOF_MASK);	//RS08 original              
		
		mtim_ms_cnt++;
		if(mtim_ms_cnt>=MTIM_MOD_VLAUE)
		{       /*0.1 sec cnt++*/
				mtim_ms_cnt=0;
				mtim_0_1_s_cnt++;
				if(mtim_0_1_s_cnt>=10)
				{      /*sec cnt++*/
						mtim_0_1_s_cnt=0;
						mtim_sec_cnt++;
						if(system_runtime_status&(1<<SYS_ERR_OUT_DISCONNECT))
								output_disconnect_cnt++;
				}
		}

		phase=g_systemCtrl.phase;
		if(phase==QI_PHASE_PING)
				if(t_wake_delay<0xff)
						t_wake_delay++;         /*t_wake counter++*/

	#ifdef _DBG
		data_log(&rec_vol,TEST_POINT_REC_VOL,1);
	#endif

		//ADC_average();
		
	#if (GPIO_DBG_PIN_1==GPIO_DBG1_MTIM1_PERIOD)
		invert_dbg_gpio(IO_DBG1);
	#endif
}

/* END Events */

#ifdef __cplusplus
}  /* extern "C" */
#endif 

/*!
** @}
*/
/*
** ###################################################################
**
**     This file was created by Processor Expert 10.3 [05.08]
**     for the Freescale Kinetis series of microcontrollers.
**
** ###################################################################
*/
