/*
 * WaRP.h
 *
 *  Created on: Mar 24, 2014
 *      Author: Jacob Postman
 *		Copyright: Revolution Robotics, Inc. 
 *		License: Createive Commons Attribution-ShareAlike 4.0 International (CC BY-SA 4.0)
 */

#ifndef WARP_H_
#define WARP_H_

#include <jsmn/jsmn.h>

// GPIO
#define GPIO_PIN_MASK            0x1Fu
#define GPIO_PIN(x)              (((1)<<(x & GPIO_PIN_MASK)))

// JSON Interface
#define JSON_MAX_TOKENS 20
#define CHAR_BUFFER_SIZE 256

#define enableUart0TxInt() UART0_C2 |= UART0_C2_TIE_MASK
#define disableUart0TxInt() UART0_C2 &= ~UART0_C2_TIE_MASK

typedef struct CHAR_BUFFER {
	char bufferArray[CHAR_BUFFER_SIZE];
	unsigned char count;
} CHAR_BUFFER;

extern int debounceCounter;

void runApplication();

/*
 * Global UART IO Queues
 * 
 * 
 */ 
#define UART_QUEUE_SIZE 256

typedef struct CHAR_QUEUE {
	char queueArray[UART_QUEUE_SIZE];
	unsigned char count;
	unsigned char in;
	unsigned char out;
	
} CHAR_QUEUE;

extern CHAR_QUEUE UartRxQueue;
extern CHAR_QUEUE UartTxQueue;

char uartQueuePut(CHAR_QUEUE *queue, char item);
char uartQueueGet(CHAR_QUEUE *queue, char *item);
void uartQueueInit(CHAR_QUEUE *queue);

/*
 * EVENT QUEUE
 * 
 * The event queue is essentially an output queue for handling
 * events and actions
 */

#define EVENT_QUEUE_SIZE 32
#define DEBOUNCE_CYCLES 1000

typedef enum {
	BTN0_DOWN = 1,
	BTN0_UP = 2,
	BTN1_DOWN = 3,
	BTN1_UP = 4,
} warpEvent_t;

typedef struct EVENT_QUEUE {
	warpEvent_t queueArray[EVENT_QUEUE_SIZE];
		unsigned char count;
		unsigned char in;
		unsigned char out;
} EVENT_QUEUE;

extern EVENT_QUEUE EventQueue;

char eventQueuePut(EVENT_QUEUE *queue, warpEvent_t event);
char eventQueueGet(EVENT_QUEUE *queue, warpEvent_t *event);
void eventQueueInit(EVENT_QUEUE *queue);

/*
 * JSON ENGINE
 * 
 * The json engine is essentially an input queue for interpreting and handling
 * incoming JSON messages.
 */

#define JSON_ENGINE_QUEUE_SIZE 8

//typedef void (*callback)(void);

typedef void (*deviceHandler_cb_t)(char* str, jsmntok_t* jsmnTkns);

typedef struct device_t {
	char* device_name;
	deviceHandler_cb_t callback;
	int   _device_index_id;
	char* commands[10];
} device_t;

typedef struct jsonCmd_t {
	int device_index;
	int action;
	char *json_str;
	jsmntok_t *json_t[];
} jsonCmd_t;

typedef struct JSON_ENGINE_QUEUE {
	jsonCmd_t queueArray[JSON_ENGINE_QUEUE_SIZE];
	unsigned char count;
	unsigned char in;
	unsigned char out;
} JSON_ENGINE_QUEUE;

extern JSON_ENGINE_QUEUE JsonEngineQueue;
extern device_t devices[10];
extern int numDevices;

void printCharArray(unsigned char *charArray, unsigned char numBytes);

char jsonEngineQueuePut(JSON_ENGINE_QUEUE *queue, jsonCmd_t json_cmd);
char jsonEngineQueueGet(JSON_ENGINE_QUEUE *queue, jsonCmd_t *json_cmd);
void jsonEngineQueueInit(JSON_ENGINE_QUEUE *queue);

#endif /* WARP_H_ */
