/*
 * File:		arm_cm4.h
 * Purpose:		Definitions common to all ARM Cortex M4 processors
 *
 * Notes:
 */

#ifndef _DEFINES_H
#define _DEFINES_H

#include <common/common.h>

/*
 * Misc. Defines
 */
#ifdef	FALSE
#undef	FALSE
#endif
#define FALSE	(0)

#ifdef	TRUE
#undef	TRUE
#endif
#define	TRUE	(1)

#ifdef	NULL
#undef	NULL
#endif
#define NULL	(0)

#ifdef  ON
#undef  ON
#endif
#define ON      (1)

#ifdef  OFF
#undef  OFF
#endif
#define OFF     (0)

/***********************************************************************/
/*
 * The basic data types
 */
typedef unsigned char		uint8;  /*  8 bits */
typedef unsigned short int	uint16; /* 16 bits */
typedef unsigned long int	uint32; /* 32 bits */

//typedef char			    int8;   /*  8 bits */
typedef short int	        int16;  /* 16 bits */
//typedef int		            int32;  /* 32 bits */

//typedef volatile int8		vint8;  /*  8 bits */
typedef volatile int16		vint16; /* 16 bits */
//typedef volatile int32		vint32; /* 32 bits */

typedef volatile uint8		vuint8;  /*  8 bits */
typedef volatile uint16		vuint16; /* 16 bits */
typedef volatile uint32		vuint32; /* 32 bits */

// function prototype for main function
int main(void);


// From FSL Power Receiver
/*Compiler switch*/
#define HW_VERSION_1_1 1
#define HW_VERSION_1_2 2
#define HW_VERSION_1_3 3
#define HW_VERSION HW_VERSION_1_2  
//#define HW_VERSION HW_VERSION_1_1  //rec hardware version2,DCDC EN: High enable DCDC Converter

/********************************************************************************************************************/
/********************************************************************************************************************/
/********************************************************************************************************************/
/*
 * 
 * 	Main system defines here
 * 	
 */
/* Use for initial Tower prototype - proto board wired into a KL46z Tower board using v1.2A1 WC Rx h/w components */
/* Leave enabled for any Kinetis port */
#define KINETIS_L_DEVICE		1		//Kinetis updates, previously from RS08.  See preprocessor settings in project

/*
 * Define one of the two below
 */
//#define TOWER_PROTO_BOARD		1		//Rx hardware tied to KL46z tower board for initial testing
#define WEARABLES_BOARD		1		//for custom board- different divider resistors

/*
 * Define supply voltage we are working with
 * Only define one!
 */
//#define V3V3_SYSTEM
#define V1V8_SYSTEM
/********************************************************************************************************************/
/********************************************************************************************************************/
/********************************************************************************************************************/
/*
 * 
 * 	End Main system defines
 * 
 * 
 */


//#define OUT_CURR_SAMPLING_USE_AMPLIFIER     // If enabled, 100X amplifer and 0.01ohm is used for current sampling, else 0.1ohm sampling resistor used.

//#define SEND_HOTIME_PK
//#define BIT_CLK_LEVEL_INVERT    /*Invert bit clock transmission level.*/
#define CTRL_ERR_AVERAGE
//#define SIG_STREN_AVERAGE
//#define ADC_10_BITS
#define ADC_10_BITS_FOR_RX_POWER
//#define DISABLE_OUTPUT_WHEN_LOW_INPUT
#define ERR_LIMIT
//#define INPUT_UNDER_VOLTAGE_CHECK
#define INPUT_OVER_VOLTAGE_CHECK
//#define OUTPUT_OPEN_CHECK
#define OUTPUT_SHORT_CHECK
//#define OUTPUT_UNDER_VOLTAGE_CHECK
//#define OUTPUT_OVER_VOLTAGE_CHECK
//#define OUTPUT_OVER_CURRENT_CHECK
#define CONTROL_NO_RESPONSE_CHECK
//#define OUTPUT_PROTECTION
#define USE_PAGE_RAM_FOR_VARIABLES

//#ifdef OUT_CURR_SAMPLING_USE_AMPLIFIER
#define ENABLE_OUT_CURR_SAMPLING_AVERAGE
//#endif

#if (HW_VERSION==HW_VERSION_1_1)
//#define POWER_GOOD_PROTECTION
#endif

/*System confirguration*/
#define PK_PREAMBLE_CNT 18      /*Packet preamble count*/

#define ADC_REC_VOL_MAX 128
#define ADC_REC_VOL_MAX_16b 512

#define ADC_OUT_CURR_CALI_OFFSET -1 // adc+=offset
#ifndef OUT_CURR_SAMPLING_USE_AMPLIFIER
#define ADC_OUT_CURR_AT_FULL_LOAD   32  // ADC value when 1A output sampled with 0.1ohm current sensing resistor
#else
#define ADC_OUT_CURR_AT_FULL_LOAD   320  // ADC value when 1A output sampled with 100x amplifier and 0.01ohm current sensing resistor
#endif

#define ADC_AVERAGE_WINDOW_SIZE_REC_VOL 8       //ADC average window size.
#define ADC_AVERAGE_WINDOW_SIZE_OUT_CURR 8       //ADC average window size.
#define ADC_AVERAGE_WINDOW_SIZE_OUT_VOL 8       //ADC average window size.


/*Debug switch*/
//#define _DBG
//#define _DBG_MASK_BIT_SEND
//#define RECEIVER_SELF_TEST_MODE /*receiver is in test mode, DC power is from external DC supply. Do not check input rectified voltage*/
//#define SIG_STREN_CONSTANT
//#define CTRL_ERR_ZERO

#define GPIO_DBG1_NONE 						0
#define GPIO_DBG1_MTIM1_PERIOD 				1
#define GPIO_DBG1_MTIM2_PERIOD 				2
#define GPIO_DBG1_MCU_WORKING    			3
#define GPIO_DBG1_ERR    					4
#define GPIO_DBG1_T_RECTIFIED    			5
#define GPIO_DBG_PIN_1 						GPIO_DBG1_NONE

#define GPIO_DBG2_NONE 						0
#define GPIO_DBG2_T_RECTIFIED    			1
#define GPIO_DBG2_ERR    					2
#define GPIO_DBG_PIN_2 						GPIO_DBG2_NONE

/* Freescale types */
typedef unsigned char       VUINT8;
typedef signed char         VINT8;
typedef unsigned short int  VUINT16;
typedef signed short int    VINT16;
typedef unsigned long int   VUINT32;


/***********************************************************************/
#endif	/* _DEFINES_H */

