/*
 * derivative.h
 *
 * Add all of the Processor Expert include files here
 * This avoids multiple include files in all other source code
 */

#ifndef DERIVATIVE_H_
#define DERIVATIVE_H_

/* non-PEx created include files */
#include "defines.h"	

/* 
 * Copy includes from the auto-generated ProcessorExpert.c file here 
 */ 
#ifdef KINETIS_L_DEVICE /* see preprocessor settings for this definition */

/*****************/
/* Kinetis L	 */
/*****************/

/* Including needed modules to compile this module/procedure */
#include "Cpu.h"
#include "Events.h"
#include "TI1.h"
#include "TimerIntLdd1.h"
#include "TU1.h"
#include "AD1.h"
#include "AdcLdd1.h"
#include "CTX.h"
#include "TI2.h"
#include "TimerIntLdd2.h"
#include "TU2.h"
#include "CALT.h"
#include "DCDC_EN.h"
/* Including shared modules, which are used for whole project */
#include "PE_Types.h"
#include "PE_Error.h"
#include "PE_Const.h"
#include "IO_Map.h"

#include "GPIO_PDD.h"


#else

/*****************/
/* RS08 Original */
/*****************/

#include "GPIO1.h"
#include "GPIO2.h"
#include "MTIM3.h"
#include "MTIM4.h"
#include "ADC1.h"
/* Include shared modules, which are used for whole project */
#include "PE_Types.h"
#include "PE_Error.h"
#include "PE_Const.h"
#include "IO_Map.h"
#endif	/* KINETIS_L_DEVICE */

/*
 * =========================================================================
 * IO Control
 * Macros created by taking advantage of the Processor Expert Code
 * =========================================================================
 */


/*
		Freescale Wireless Qi Charging code support functions
		Author: Freescale
 *      License / Copyright: Please refer to Freescale Semiconductor, Inc.
*/

/*
 * Coil Clamping control - CTX Signal (PTE19)
 * ===================================================================
 * CTX_SetVal(NULL);
 * CTX_ClrVal(NULL);
 * CTX_GetVal(NULL);
 * CTX_NegVal(NULL);
 */
#define CTX_ON						CTX_SetVal(NULL);	//GPIOE_PDOR |= GPIO_PDOR_PDO(CTX_PORT_MASK);			/* CTX Enable - Set */
#define CTX_OFF						CTX_ClrVal(NULL);	//GPIOE_PDOR &= ~GPIO_PDOR_PDO(CTX_PORT_MASK);			/* CTX Disable - Clear */
#define CTX_INVERT					CTX_NegVal(NULL);	//GPIOE_PTOR |= GPIO_PDOR_PDO(CTX_PORT_MASK);			/* CTX Toggle */
#define CTX_READ					CTX_GetVal(NULL);	//(GPIOE_PDOR & CTX_PORT_MASK);							/* CTX Read */

/*
 * Inverter clamp Signal - CALT (PTE20)
 * ===================================================================
 * CALT_SetVal(NULL);
 * CALT_ClrVal(NULL);
 * CALT_GetVal(NULL);
 * CALT_NegVal(NULL);
 * 
 * Doesn't seem to be used in v2.0
 */
#define CALT_ON						CALT_SetVal(NULL);	
#define CALT_OFF					CALT_ClrVal(NULL);	
#define CALT_INVERT					CALT_NegVal(NULL);	
#define CALT_READ					CALT_GetVal(NULL);	

/*
 * DC-DC Converter enable control (PTE21)
 * ===============================================================
 * DCDC_EN Signal
 * DCDC_EN_SetVal(NULL);	//sets to 1 
 * DCDC_EN_ClrVal(NULL);	//sets to 0
 * DCDC_EN_GetVal(NULL);	//read it
 * DCDC_EN_NegVal(NULL);	//negate it
 */
#if (HW_VERSION==HW_VERSION_1_1)
#define DCDC_EN_ON 					DCDC_EN_SetVal(NULL);	//GPIOE_PDOR |= GPIO_PDOR_PDO(DCDC_EN_PORT_MASK);			/* DCDC_EN - Set */
#define DCDC_EN_OFF 				DCDC_EN_ClrVal(NULL);	//GPIOE_PDOR &= ~GPIO_PDOR_PDO(DCDC_EN_PORT_MASK);			/* DCDC_EN - Clear */
#define DCDC_EN_INVERT				DCDC_EN_NegVal(NULL);
#elif (HW_VERSION==HW_VERSION_1_2)
#define DCDC_EN_ON 					DCDC_EN_ClrVal(NULL);	//GPIOE_PDOR &= ~GPIO_PDOR_PDO(DCDC_EN_PORT_MASK);			/* DCDC_EN - Clear */
#define DCDC_EN_OFF 				DCDC_EN_SetVal(NULL);	//GPIOE_PDOR |= GPIO_PDOR_PDO(DCDC_EN_PORT_MASK);			/* DCDC_EN - Set */
#define DCDC_EN_INVERT				DCDC_EN_NegVal(NULL);
#else
#error "No Hardware Version Defined!!"
#endif
		

/*
 * !!!!!!!!!!!!!!!!!! These are for KL46z board only - for initial protoyping debug.  !!!!!!!!!!!!!!!!!!!!!
 * !!!!!!!!!!!!!!!!!! The new board may use different IO to drive LEDs !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
 * 
 */
#ifdef TOWER_PROTO_BOARD

#define LED0_EN { (PORTA_PCR16 = PORT_PCR_MUX(1)); 	(GPIOA_PDDR |= 1<<16); 	(GPIOA_PSOR = (1<<16)); }/* enabled but off */
#define LED1_EN { (PORTA_PCR17 = PORT_PCR_MUX(1)); 	(GPIOA_PDDR |= 1<<17); 	(GPIOA_PSOR = (1<<17)); }/* enabled but off */
#define LED2_EN { (PORTB_PCR8 = PORT_PCR_MUX(1));  	(GPIOB_PDDR |= 1<<8);	(GPIOB_PSOR = (1<<8)); }/* enabled but off */
#define LED3_EN { (PORTA_PCR5 = PORT_PCR_MUX(1));  	(GPIOA_PDDR |= 1<<5); 	(GPIOA_PSOR = (1<<5)); }/* enabled but off */

#define LED0_TOGGLE (GPIOA_PTOR = (1<<16))
#define LED1_TOGGLE (GPIOA_PTOR = (1<<17))
#define LED2_TOGGLE (GPIOB_PTOR = (1<<8))
#define LED3_TOGGLE (GPIOA_PTOR = (1<<5))

#define LED0_OFF (GPIOA_PSOR = (1<<16))
#define LED1_OFF (GPIOA_PSOR = (1<<17))
#define LED2_OFF (GPIOB_PSOR = (1<<8))
#define LED3_OFF (GPIOA_PSOR = (1<<5))

#define LED0_ON (GPIOA_PCOR = (1<<16))
#define LED1_ON (GPIOA_PCOR = (1<<17))	
#define LED2_ON (GPIOB_PCOR = (1<<8))
#define LED3_ON (GPIOA_PCOR = (1<<5))

#else

/* Custom LEDs */
#define LED0_EN { ; }
#define LED1_EN { ; }
#define LED2_EN { ; }
#define LED3_EN { ; }

#define LED0_TOGGLE ;
#define LED1_TOGGLE ;
#define LED2_TOGGLE ;
#define LED3_TOGGLE ;

#define LED0_OFF ;
#define LED1_OFF ;
#define LED2_OFF ;
#define LED3_OFF ;

#define LED0_ON ;
#define LED1_ON ;
#define LED2_ON ;
#define LED3_ON ;

#endif  /* TOWER_PROTO_BOARD */

#endif /* DERIVATIVE_H_ */
