/*
 * device_Buzzer.c
 *
 *  Created on: Dec 16, 2014
 *      Author: Jacob Postman
 *		Copyright: Revolution Robotics, Inc. 
 *		License: Createive Commons Attribution-ShareAlike 4.0 International (CC BY-SA 4.0)
 */

#include "Cpu.h"
#include "Events.h"
#include "UART0.h"
#include "PWM1.h"
/* Including shared modules, which are used for whole project */
#include "PE_Types.h"
#include "PE_Error.h"
#include "PE_Const.h"
#include "IO_Map.h"

#include <common/common.h>
#include <uart.h>
#include <jsmn/jsmn.h>
#include <json/json.h>

#include "../WaRP_DB.h"
#include "devices/device_Buzzer.h"

void buzzerHandler(char* jsonStr, jsmntok_t *jsmnTkns){
	
	int jsmnTknIndex, j, k, v;
	
	jsonParseState state = START;
	int objTknCnt = 0;
	int skipEndIndex = 0;
	int valFound = 0;
	
	for(jsmnTknIndex=0, j=1; j > 0; jsmnTknIndex++, j--){
		jsmntok_t *t = &jsmnTkns[jsmnTknIndex];

		if (t->type == JSMN_ARRAY || t->type == JSMN_OBJECT)
			j += t->size;

		switch(state){
		case START:
			if (t->type != JSMN_OBJECT){
				jsonSysErr("JSON Error: root element must be an object.");
				return;
			}

			state = KEY;
			objTknCnt = t->size;

			if (objTknCnt == 0)
				state = STOP;

			if (objTknCnt % 2 != 0){
				jsonSysErr("JSON Error: object must have even number of children.");
				return;
			}
			break;
		case KEY:
			objTknCnt--;

			if(t->start <= skipEndIndex)
				break;

			if (t->type != JSMN_STRING){
				jsonSysErr("JSON Error: key elements must be strings.");
				return;
			}

			state = SKIP;

			for(k = 0; k < numDevices * sizeof(devices)/sizeof(device_t); k++){
				if(json_token_streq(jsonStr, t, devices[k].device_name)){
					state = VALUE;
					break;
				}
			}
			break;

		case SKIP:
			objTknCnt--;
			if(t->type != JSMN_STRING && t->type != JSMN_PRIMITIVE){
				// If the element to be skipped is an array or object, also skip it's children
				//token_index += t->size;
				if(t->end > skipEndIndex)
					skipEndIndex = t->end;
			}

			state = KEY;

			if (objTknCnt == 0)
				state = STOP;
			break;

		case VALUE:
					if (t->type != JSMN_STRING && t->type != JSMN_PRIMITIVE){
						jsonSysErr("JSON Error: value elements must be strings or primitives.");
						return;
					}
					for (v = 0; v < sizeof(Buzzer_dev.commands)/sizeof(char*); v++)
					{
						if (json_token_streq(jsonStr, t, Buzzer_dev.commands[v]))
						{
							valFound = 1;
							break;
						}
					}
					if(valFound){
						switch(v){
							case(0):
								PWM1_Enable();
								break;
							case(1):
								PWM1_Disable();
								break;
							default:
								break;
						}
						
						//json_cmd.device_index = 0;
						//json_cmd.action = 0;
						//jsonEngineQueuePut(&JsonEngineQueue, json_cmd);
						//mmaDbgPrintf("Found %s while searching for %s\n", subStr, MMA9553L_dev.commands[v]);
					}else{
						printf("{\"SYSMSG\":\"JSON Error: unknown JSON value: %s\"}\n", json_token_tostr(jsonStr, t));
						//	return -1;
					}
				objTknCnt--;
				state = KEY;

				if (objTknCnt == 0)
					state = STOP;

				break;
		case STOP:
			// Just consume the tokens
			break;

		default:
			jsonSysErr("JSON Error: Invalid state.");
			return;
		}
	}
}

struct device_t Buzzer_dev = {
	.device_name = "BUZZER",
	.callback = buzzerHandler,
	.commands =	{
			"ON",		// 0
			"OFF",		// 2
	}
};

