/*
 * fsl_pwr_rcvr.h
 *
 *  Created on: Dec 18, 2014
 *      Author: Freescale
 *      License / Copyright: Please refer to Freescale Semiconductor, Inc.
 */

#ifndef FSL_PWR_RCVR_H_
#define FSL_PWR_RCVR_H_

#ifdef TOWER_PROTO_BOARD

/* 
 * 
 * Debug
 * 
 */
#define EXTERNAL_THERMAL
#define OUTPUT_OVER_TEMPERATURE_CHECK
#define OUTPUT_OPEN_CHECK
//#define OUTPUT_OVER_VOLTAGE_CHECK		//remove for initial debug on v1.2A hardware
//#define OUTPUT_OVER_CURRENT_CHECK		//remove for initial debug on v1.2A hardware
#define OUTPUT_UNDER_VOLTAGE_CHECK

#else
/*
 * Original
 */
#define EXTERNAL_THERMAL
#define OUTPUT_OVER_TEMPERATURE_CHECK
#define OUTPUT_OPEN_CHECK
#define OUTPUT_OVER_VOLTAGE_CHECK
#define OUTPUT_OVER_CURRENT_CHECK
#define OUTPUT_UNDER_VOLTAGE_CHECK

#endif

/* ADC channel definition */
#ifdef KINETIS_L_DEVICE

	/*
	 * For Kinetis, let's use the PEx components to define channels, for more portability btw applications
	 */

/* These get read in function system_monitor() */
/* TODO:  Update the defines below, throughout the code, to match the PEx names/index, i.e. change ADC_CH_DC_OUT_VOLT to ADC_5VOUT_AD2, etc */ 
/* Unfortunately the ChannelToPin array is defined as a static const in another file, which means we can't use it here.  */
/* TODO: Figure out a way to tie this to the PEx files instead of hard coding the channel */ 
//#define ADC_CH_DC_OUT_VOLT 		ChannelToPin[AdcLdd1_ADC_5VOUT_AD2]		//index: 0, value: 1 (ADC channel number)
//#define ADC_CH_DC_OUT_CURR 		ChannelToPin[AdcLdd1_ADC_5VOUT_I_AD3]   //index: 1, value: 2 (ADC channel number)
//#define ADC_CH_VBUS				ChannelToPin[AdcLdd1_VBUS_AD1]			//index: 2, value: 5 (ADC channel number)
#define ADC_CH_DC_OUT_VOLT    		1   /*PTE16/ADC0_SE1 - Output DC voltage - ADC_5VOUT_AD2 */
#define ADC_CH_DC_OUT_CURR    		2   /*PTE18/ADC_SE2 - Output DC current - ADC_5VOUT_I_AD3 */
#define ADC_CH_VBUS   				5 	/*PTE17/ADC_SE5a - Rectified voltage - VBUS_AD1 */
/* See Also:  ChannelToPin[] */

#else 

	/* RS08 original - ADC Channel numbers */
#define ADC_CH_VBUS   				11   /*PTC3/ADP11 - Rectified voltage - VBUS_AD1 */
#define ADC_CH_DC_OUT_CURR    		1    /*PTA1/ADP1 - Output DC current - VBUS_I_AD3*/
#define ADC_CH_DC_OUT_VOLT    		6    /*PTB2/ADP6 - Output DC voltage - 5VOUT_AD2 */
#define ADC_CH_TEMP       			26   /*System temerature*/
#define ADC_CH_ntc_thermistor		5    /*PTB1/ADP5 - external ntc temerature sensor - 3V3 */

#endif /* #ifdef KINETIS_L_DEVICE */


#ifdef KINETIS_L_DEVICE

	/* The IO definitions below are used primarily outside of the state machine required for charging.  Used for debugging mainly */
	//IO_COM1 - Now using macros, i.e., CTX_ON, CTX_OFF, CTX_TOGGLE, see derivative.h

#else

/*GPIO definition*/
/*PTB*/
#define IO_COM1 		3	//CTX
#define IO_DISCH_EN 	4	//CALT
#define IO_DC_EN 		5	//DCDC_EN
/*PTC*/
#define IO_DC_PG 		2	//RX_EN - not used for KL16 port
#define IO_DBG1 		1	//USB_EN - not used for KL16 port
#define IO_DBG2 		0	//Debug pin

#define IO_USB_EN 			1	//unused
#define IO_RX_EN 			2	//PTC3 on RS08, VBUS_AD1.  not using?
#define IO_ADAPTOR_DETECT	0	//PTB0 on RS08, not used for KL16 port
//PTB1:3V3(RS08) / Unused(Kinetis KL16)

/*GPIO definition*/
const VUINT8 GPIO_Table[]={   /*GPIO   - Function         - I/O*/
        (1<<4)+3,             /*PTB3   - com              - out*/ 	//CTX
        (1<<4)+4,             /*PTB4   - discharge_enable - out*/	//CALT
        (1<<4)+5,             /*PTB5   - dc-dc_enable(set low,version1) ,(set high,version2) - out*/	//DCDC_EN
        (2<<4)+2,             /*PTC2   - dc-dc_PG         - in */	//5VOUT_AD2/ADC_V_WLREG_OUT
        (2<<4)+1,             /*PTC1   - debug1           - out*/	//3v3/unused kinetis kl16
        (2<<4)+0,             /*PTC0   - debug2           - out*/	//debug pin
        (1<<4)+0,             /*PTB0   - adpator detect   - in*/	
};
#endif	/* #ifdef KINETIS_L_DEVICE */


//#pragma DATA_SEG DEFAULT

VUINT8 outBufSts;        /*Packet output buffer status. 0/1: empty or full.*/
VUINT8 bitClkFlag;       /*bit clock period. 0/1: first half period/second half period */


VUINT8 g_status;                             /*System running status*/
VUINT8 g_ErrorStatus;                        /*System error status*/
VUINT8 mtim_ms_cnt;                   /*millisecond count*/
//VUINT8 time_point_ms;                   /*millisecond timepoint*/
VUINT8 mtim_0_1_s_cnt;                /*0.1sec count*/
VUINT8 mtim_sec_cnt;                /*10 sec count*/
VUINT8 time_point_ms;               /*0.1sec timepoint*/ 
VUINT8 time_point_0_1_s;               /*0.1sec timepoint*/ 
VUINT8 t_wake_delay;

T_system_control g_systemCtrl;          /*system control data*/
T_packet_tx_control g_pkTxCtrl;         /*communication control data*/
T_control_point g_ctrlPoint_actual;
T_control_point g_ctrlPoint_desired;


VUINT8 sig_stren;                       /*signal strength*/
VUINT8 rec_vol;                         /*Vbus instant voltage*/
VUINT8 rec_vol_average;                 /*Vbus average voltage*/
VUINT8 out_vol;                         /*output voltage*/
VUINT8 out_cur;                         /*output current*/
VUINT8 sys_temp;                        /*system temperature*/
VINT8 rec_err;                          /*Vbus err*/
VINT8 out_err;                          /*DC-5v err*/
VUINT8 rec_vol_expected;                /*debug use*/
#ifdef POWER_GOOD_PROTECTION
VUINT8 dcdc_power_good;                      /*DC-DC power good signal*/
#endif
VUINT8 rec_pwr_percent;

VUINT8 end_power_code;           /*end power code*/
VUINT8 system_error;            /*system error status*/
VUINT16 system_runtime_status;           /*output status*/

VUINT8 rec_over_vol_cnt;
VUINT8 output_short_cnt;
VUINT8 output_disconnect_cnt;
VUINT8 output_over_curr_cnt;
VUINT8 output_over_vol_cnt;
VUINT8 output_under_vol_cnt;
VUINT8 output_over_temper_cnt;

//VUINT16 rec_under_voltage_cnt;
//VUINT16 rec_vol_stable_cnt;

#ifdef ADC_10_BITS
VUINT16 rec_vol_16b;
VUINT16 out_vol_16b;
VUINT16 out_cur_16b;
VUINT16 sys_temp_16b;

VINT16 rec_err_16b;
VINT16 out_err_16b;
#endif

#ifdef ADC_10_BITS_FOR_RX_POWER
VUINT16 out_vol_10b;
VUINT16 out_cur_10b;
#endif

VUINT8 curr_sense_offset;

#ifndef KINETIS_L_DEVICE
#ifdef USE_PAGE_RAM_FOR_VARIABLES	//was for RS08 originally
#pragma DATA_SEG __PAGED_SEG PAGED_RAM
#endif
#endif //#ifndef KINETIS_L_DEVICE

VUINT8 rec_vol_buf[ADC_AVERAGE_WINDOW_SIZE_REC_VOL];    /*rectifier voltage buffer for ADC average*/
VUINT8 rec_vol_buf_idx;   /*buffer index to write*/
VUINT16 out_vol_buf[ADC_AVERAGE_WINDOW_SIZE_OUT_VOL];    /*rectifier voltage buffer for ADC average*/
VUINT8 out_vol_buf_idx;   /*buffer index to write*/

#ifdef ADC_10_BITS_FOR_RX_POWER
VUINT16 out_cur_10b_average;                 /*output current average*/
VUINT16 out_vol_average;
#endif


VUINT8 usb_charger_toggle;


#ifdef ENABLE_OUT_CURR_SAMPLING_AVERAGE
VUINT16 out_cur_10b_buf[ADC_AVERAGE_WINDOW_SIZE_OUT_CURR];    /*rectifier voltage buffer for ADC average*/
VUINT8 out_cur_10b_buf_idx;                                         /*buffer index to write*/
#endif

long sdwReceivedPower;	// Signed in case a coefficient is negative
VUINT32 dwOutputPower;
VUINT32 sdwTemp;
VUINT32 dwReceivedPower;
VUINT32 dwVoltageData ;
VUINT32 dwCurrentData ;


VUINT32 debug_dwVoltageData ;
VUINT32 debug_dwCurrentData ;
VUINT32 debug_dwOutputPower;
VUINT32 debug_dwReceivedPower;

#ifdef KINETIS_L_DEVICE

/*
 * From PEx components for KL16 port
 */
extern word AD1_OutV[AD1_SAMPLE_GROUP_SIZE]; /* Sum of measured values */

#endif /* #ifdef KINETIS_L_DEVICE */

#ifdef _DBG
VUINT8 TestPoint;
VUINT8 logBufCnt;
VUINT8 logBuf[64];

/*Debug data*/
VUINT8 d_pkTxCnt;
VUINT8 d_byteTxCnt;
VUINT8 d_bitTxCnt;
VUINT8 d_bitClkCnt;                  

/*Bit0: 1: enable ping test.
*/
VUINT8 d_testFlag;

VUINT16 time_point_buf[QI_TS_SIZE];              /*Qi timing stamp record buffer*/
#endif

#ifndef KINETIS_L_DEVICE
#ifdef USE_PAGE_RAM_FOR_VARIABLES	//for RS08
#pragma DATA_SEG DEFAULT
#endif
#endif //#ifndef KINETIS_L_DEVICE

void ntc_thermistor_sample(void);
void rx_enable(void);
void rx_disable(void);
VUINT8 detect_adapter(void);
void usb_enable(void);
void usb_disable(void);

#endif /* FSL_PWR_RCVR_H_ */
