
/* Including needed modules to compile this module/procedure */
#include "Cpu.h"
#include "Events.h"
#include "UART0.h"
/* Including shared modules, which are used for whole project */
#include "PE_Types.h"
#include "PE_Error.h"
#include "PE_Const.h"
#include "IO_Map.h"

#include "WaRP_DB.h"
#include <common/common.h>
#include <uart.h>
#include <jsmn/jsmn.h>
#include <json/json.h>


void jsonSysErr(char *str){
	printf("{\"SYSMSG\":\"%s\"}\n", str);
}

int json_token_streq(char *json_str, jsmntok_t *t, char *s)
{
    return (strncmp(json_str + t->start, s, t->end - t->start) == 0
            && strlen(s) == (size_t) (t->end - t->start));
}

char * json_token_tostr(char *json_str, jsmntok_t *t)
{
    json_str[t->end] = '\0';
    return json_str + t->start;
}

char *KEYS[] = { 		// KEY_ID
		"MMA9553L",		// 0
};

char *CMDS[][10] = {
		{"GET", "SET"} // 0 - MMA9553L
};

int jsonParseLine(char * jsonStr){

	int jsmnTknIndex, j;
	int k;
	int jsmnTknCnt;

	jsmn_parser p;
	jsmntok_t jsmnTkns[JSON_MAX_TOKENS];
	jsmn_init(&p);
	jsmnTknCnt = jsmn_parse(&p, jsonStr, strlen(jsonStr), jsmnTkns, JSON_MAX_TOKENS);
	
	if(jsmnTknCnt < 0){
		jsonSysErr("JSON Parser Error");
		return -1;
	}else if(jsmnTknCnt > JSON_MAX_TOKENS){
		//TODO: improve custom jsonSysErr message 
		printf("{\"SYSMSG\":\"JSON Error: %d JSON tokens is greater than max allowed:%d\"}", 
				jsmnTknCnt, JSON_MAX_TOKENS);
		return -1;
	}	
	// Parse the JSMN tokens for keys/values and assign to handlers
	jsonParseState state = START;
	int objTknCnt = 0;
	int skipEndIndex = 0;

	for(jsmnTknIndex=0, j=1; j > 0; jsmnTknIndex++, j--){

		jsmntok_t *t = &jsmnTkns[jsmnTknIndex];

		if (t->type == JSMN_ARRAY || t->type == JSMN_OBJECT)
			j += t->size;

		switch(state){
		case START:
			if (t->type != JSMN_OBJECT){
				jsonSysErr("JSON Error: root element must be an object.");
				return -1;
			}

			state = KEY;
			objTknCnt = t->size;

			if (objTknCnt == 0)
				state = STOP;

			if (objTknCnt % 2 != 0){
				jsonSysErr("JSON Error: object must have even number of children.");
				return -1;
			}
			break;
		case KEY:
			objTknCnt--;

			if(t->start <= skipEndIndex)
				break;

			if (t->type != JSMN_STRING){
				jsonSysErr("JSON Error: key elements must be strings.");
				return -1;
			}

			state = SKIP;

			//for(k = 0; k < nextDevice * sizeof(devices)/sizeof(device_t); k++){
			//max_num_devices = sizeof(devices)/sizeof(device_t);
			for(k = 0; k < numDevices; k++){
				if(json_token_streq(jsonStr, t, devices[k].device_name)){
					//printf("Sending to device callback: %s\n", devices[k].device_name);
					(*devices[k].callback)(jsonStr, jsmnTkns);
					
					//Since more than one device can receive this message, do not break.
				}
			}

			break;

		case SKIP:
			objTknCnt--;
			if(t->type != JSMN_STRING && t->type != JSMN_PRIMITIVE){
				// If the element to be skipped is an array or object, also skip it's children
				if(t->end > skipEndIndex)
					skipEndIndex = t->end;
			}

			state = KEY;

			if (objTknCnt == 0)
				state = STOP;
			break;

		case STOP:
			// Just consume the tokens
			break;

		default:
			jsonSysErr("JSON Error: Invalid state.");
			return -1;
		}
	}
	
	return 0;
}
