#include "params.h"
#include "prx_lib.h"

/*Qi timing constrains table*/
const VUINT8 g_timing_Tbl[]=
{
        /*Ping phase*/
        40,     /*t_wake*/

        /*ide&cfg phase*/
        7,      /*t_silent*/

        /*pwr_trans phase*/
        5,      /*short t_delay*/
        215,     /*long t_delay*/
        25,     /*t_control.*/
};

/*System parameters and configurations*/
const T_system_params g_SysParams={

        /*WPC version*/
        (1<<4)+1,

        /*Receiver information*/
        0x0028,          /*manufCode, FSL:0x28 */
        0x76543210,      /*devId*/
        10,              /*maxPwr.*/
        
        /*HW configuration*/
        
        /*execution timer*/
        8,             /*Trectified, rec_power packet is sent every 8 err packets. It's around 2sec, =8*(25+240)=2120ms, packet length:25ms, max packet delay: 240ms*/

#ifdef ADC_10_BITS
        /*charging parameter -- 16bit*/
        203,         	/*5.0v, rectified_voltage_stable_level_16b*/
        264,        	/*7.5v, rectified_voltage_expected_16b*/
        672,            /*max_DC_voltage_16b*/
        300,            /*1.0A, max_DC_current_16b*/
        672,            /*5.0v, out_voltage_expected_16b*/
#endif



#ifdef KINETIS_L_DEVICE
        
        /********************************************************************************************************
         * Let's use constants instead of hard-coding values - more portable 
         * 
         * VBUS_AD1	->	Rectifier voltage	->	0-20V Range 
         * ------------------------------------------------------------------------------------ 
         * 	v1.2A1 hardware uses divider that is VBUS*(1/20) -> @0-20V, Measured range: 0-2V
         * 	v2.0 hardware uses divider that is  VBUS*(1/6) -> @0-20V, Measured range: 0-3.3V 
         * 
         * VBUS_AD2 ->  Output voltage 		-> 	0-5V Range
         * -------------------------------------------------------------------------------------
         * 	v1.2A1 & v2.0 hardware uses divider that is VOUT*(1/3) -> @0-5V, Measured range: 0-1.5V
         * 
         * VBUS_AD3 ->	Output current		-> 0-2V Range --> up to 1A measured across 0.02R, gain of 50 for MAX9634FERS+
         * See R_CURRENT_SENSE_VALUE & I_SENSE_GAIN definitions in prx_lib.h
         * -----------------------------------------------------------------------------------------------------------------
         * 	
         ******************************************************************************************************************/
        
        /* 4.5V:  Ping voltage threshold after startup */
        VBUS_ADC_INT_COUNT_4V5,         
        /* Expected input level after DC on */
        VBUS_ADC_INT_COUNT_7V5, 
        /* DC-DC stable level */
        VBUS_ADC_INT_COUNT_6V0,
        /* return difference for Vbus from unstable to stable level */
        VBUS_ADC_INT_COUNT_7V0,
        
        /* expected output voltage */
        VOUT_ADC_INT_COUNT_5V0,        
        /* max rectified voltage during power transfer.  194*/
        VBUS_ADC_INT_COUNT_15V0,        
        /* max output voltage */
        VOUT_ADC_INT_COUNT_5V5,        
        /* min output voltage. */
#ifdef TOWER_PROTO_BOARD
        /* workaround using the proto hardware - VOUT has an additional path to GND thru a 470R, so there is voltage drop.  compensate here. */
        /* You can get around this by commenting out OUTPUT_UNDER_VOLTAGE_CHECK to ignore the additional checks */
        VOUT_ADC_INT_COUNT_4V5/4,	  
#else
        VOUT_ADC_INT_COUNT_4V5,
#endif
        
        /* max output current. */
        VOUT_ADC_I_MAX_CURRENT_COUNT_1A5,	//450 originally    
        
#ifdef TOWER_PROTO_BOARD
        /* min output current. */
        1, 
#else
        VOUT_ADC_I_MIN_CURRENT_COUNT_50MA,	//here, this calculates to count of 3; For v1.2A, this value was 0x1; for v2.0, this is 10.  Current sense circuit is different
#endif
        
        
        /* System Temperature Threshold */
        
#ifdef TOWER_PROTO_BOARD
        
        /*
         * Use hardcoded values from the original v1.2A1 code
         */
        
        // RS08 chip temp sensor slope, 25oc-125oc, m=3.638 mv/oC, temp sensor voltage, Vtemp25oc=1.396v
        // temp=25 - (Vtemp-Vtemp25)/m => Vtemp=Vtemp25-(temp-25)*m, adc=Vtemp/3.3*256

        /*  --- |------------|---------|
         *  ADC |Temperature | Loading |
         *  --- |------------|---------|
         * 108  | 25
         * 102  | 47
         * 94   | 75         | 1A
         * 93   | 80         | 
         * 91   | 85
         * 90   | 90
         * 89   | 95
         * 87   | 100
         */
        93,   /*80c*/
        
        /* V1.2A1 didn't have FOD but add these to match up with header and turn off FOD compile options to test */
        10, /*fod quadratic_coefficient.*/
        139,/*fod linear coefficient.*/
        621,/*fod offset mw.*/

        /*system error time-out*/
        10,  /*time_out_of_rec_over_vol, unit: 0.1s*/
        10,  /*time_out_of_out_vol_out_of_range, unit: 0.1s*/
        10,  /*time_out_of_out_short, unit: 0.1s*/
        5,   /*time_out_of_out_over_curr, unit: 0.1s*/
        60,  /*time_out_of_out_disconnect, unit: 1s, if sampled output current is zero and it lasts for certain time, then end power is sent.*/
		5,	 /*time_out_of_out_over_temperature, unit: 0.1s*/        

#else
		
		/*
		 * Keep values from 2.0 hardware
		 */
		
        /*  --- |------------|---------|
         *  ADC |Temperature | Loading |
         *  --- |------------|---------|
         * 108  | 25
         * 102  | 47
         * 94   | 75         | 1A
         * 93   | 80         | 
         * 91   | 85
         * 90   | 90
         * 89   | 95
         * 87   | 100
         */
        233,  /* 80c RT=9.3285255*100/(100+9.33) ,//93, 80c*/

        10, /*fod quadratic_coefficient.*/
        139,/*fod linear coefficient.*/
        621,/*fod offset mw.*/

        /*system error time-out*/
        10,  /*time_out_of_rec_over_vol, unit: 0.1s*/
        10,  /*time_out_of_out_vol_out_of_range, unit: 0.1s*/
        10,  /*time_out_of_out_short, unit: 0.1s*/
        5,   /*time_out_of_out_over_curr, unit: 0.1s*/
        60, /*time_out_of_out_disconnect, unit: 1s, if sampled output current is zero and it lasts for certain time, then end power is sent.*/
		5,	 /*time_out_of_out_over_temperature, unit: 0.1s*/
		
#endif
        
        
        
        
#else
        /************************************/
        /* RS08 original ADC setup values */
        /************************************/
        
        
        /*charging parameter*/
        /* ADC 8bit, Vref=Vdd=3.3v
         *  --- |--------
         *  ADC |  rec_vol
         *  --- |--------
         *  25     3.3v
         *  29     3.75v=7.5/2
         *  39     5.0v
         *  50     6.5v
         *  54     7.0v
         *  58     7.5v 
         *  62     8.0v
         *  66     8.5v
         *  70     9.0v
         *  77     10.0v
         *  116    15.0v
         *  128    16.5v
         *  155    20.0v
         */        
        
        58,    //34//adc=4.5/6/3.3*256= 58 for version 2/*rectified_voltage_stable_level_startup, 4.5v, adc=4.5/10/3.3*256*/
        80,    //58//adc=6.2/6/3.3*256 = 80 for  version 2/* rectified_voltage_expected,7.5v, adc=Vbus/10/3.3*256*/ 
        48,    /*dc_dc_stable_level, for MP28253, 6.0v. 6.2/10/3.3*256*/
        54,    /*dc_dc_stable_level_ret_dif, 7.0v. */

        116,   /*out_voltage_expected, 5.0v, adc=5.0*0.3/3.3*256*/
        194,   //(VBUS =15.0)*(1/6)/3.3*256*///116, /*max_rec_voltage, (VBUS = 15.0)*0.1/3.3*256*/      
        127,   //127 5.5*0.298/3.3*256,   128/*max_out_voltage, 5.5v, adc=5.5*0.3/3.3*256 */
        104,   /*min_out_voltage, 4.5v*/

        /*  ------------|--------------
         *  10 bit      |
         *  ADC value   |  out_current
         *  ------------|--------------
         *  Calc | Real |
         *  ---  | ---  |
         *  0    |      |   <50mA        
         *  1    |      |   50mA        
         *  16   |  3   |   0.5A
         *  32   |  5   |   1.0A
         *  45   |  8   |   1.5A
         */

        //11,    /*max_out_current, 1.5A, for 0.1ohm current sensing resistor, for max 1.5A current, vol=0.15v, adc=0.15/3.3*256*/
        450,//0.02*1.5*50/3.3*1024, 45,    /*10bit ADC, max_out_current, 1.5A, for 0.1ohm current sensing resistor, for max 1.5A current, vol=0.15v, adc=0.15/3.3*1024*/
        10,//0.02*0.05*50/3.3*1024, 1,     /*min_out_current, 50mA, for 0.1ohm current sensing resistor, for min 0.05A current, vol=0.005v, adc=0.005/3.3*1024*/

        
        //System temerature threshold
        // RS08 chip temp sensor slope, 25oc-125oc, m=3.638 mv/oC, temp sensor voltage, Vtemp25oc=1.396v
        // temp=25 - (Vtemp-Vtemp25)/m => Vtemp=Vtemp25-(temp-25)*m, adc=Vtemp/3.3*256

        /*  --- |------------|---------|
         *  ADC |Temperature | Loading |
         *  --- |------------|---------|
         * 108  | 25
         * 102  | 47
         * 94   | 75         | 1A
         * 93   | 80         | 
         * 91   | 85
         * 90   | 90
         * 89   | 95
         * 87   | 100
         */
        233,  /* 80c RT=9.3285255*100/(100+9.33) ,//93, 80c*/

        10, /*fod quadratic_coefficient.*/
        139,/*fod linear coefficient.*/
        621,/*fod offset mw.*/

        /*system error time-out*/
        10,  /*time_out_of_rec_over_vol, unit: 0.1s*/
        10,  /*time_out_of_out_vol_out_of_range, unit: 0.1s*/
        10,  /*time_out_of_out_short, unit: 0.1s*/
        5,   /*time_out_of_out_over_curr, unit: 0.1s*/
        60, /*time_out_of_out_disconnect, unit: 1s, if sampled output current is zero and it lasts for certain time, then end power is sent.*/
		5,	 /*time_out_of_out_over_temperature, unit: 0.1s*/
		
#endif
		
};

