#ifndef __Param_H
#define __Param_H

#include "defines.h"

/*System parameters*/
typedef struct{
        /*The revision of Wireless Power Transfer in this implementation.*/
        VUINT8 WPC_revision;     /*bit[7:4]: major version, bit[3:0]: minor version. */

        /*Receiver information*/
        VUINT16 manufCode;      /*manufacture code*/
        VUINT32 devId;      /*device identifier*/

        VUINT8 maxPwr;                 /*Max power receiver expected at the output of the rectifier.*/

        /*HW configuration*/
        
        /*execution timer*/
        VUINT8 Trectified;         /*rectified power packet delay, sent every n err packets*/

        /*charging parameter*/
#ifdef ADC_10_BITS
        VUINT16 rectified_voltage_stable_level_16b;
        VUINT16 rectified_voltage_expected_16b;
        VUINT16 max_DC_voltage_16b;
        VUINT16 max_DC_current_16b;
        VUINT16 out_voltage_expected_16b;
#endif

        VUINT8 rectified_voltage_stable_level_startup;  /*Ping voltage threshold after startup.*/
        VUINT8 rectified_voltage_expected;              /*Expected input level after DC on.*/
        VUINT8 dc_dc_stable_level;                         /*DC-DC stable level*/
        VUINT8 dc_dc_stable_level_ret_dif;              /*return difference for Vbus from unstable to stable level*/
        VUINT8 out_voltage_expected;

        VUINT8 max_rec_voltage;                         /*max rectified voltage during power transfer.*/
        VUINT8 max_out_voltage;                         /*max output voltage.*/
        VUINT8 min_out_voltage;                         /*min output voltage.*/
        VUINT16 max_out_current;                         /*max output current.*/
        VUINT8 min_out_current;                         /*min output current.*/

        /*System temerature threshold*/
        VUINT16 max_temperture;

        VUINT16 configure_quadratic_coefficient;	/*fod quadratic_coefficient.*/
        VUINT16 configure_linear_coefficient;		/*fod linear coefficient.*/
        VUINT16 configure_offset_mw;				/*fod offset mw.*/

        /*system error time-out, unit: ms*/
        VUINT8 time_out_of_rec_over_vol;               /* End power when rectified voltage is out of range for a certain time. */
        VUINT8 time_out_of_out_vol_out_of_range;  /* End power when output voltage is out of range for a certain time. */
        VUINT8 time_out_of_out_short;                    /* End power when output is short for a certain time. */
        VUINT8 time_out_of_out_over_curr;             /* End power when out over current for a certain time. */
        VUINT8 time_out_of_out_disconnect;            /* End power when output current is low for a certain time. */
		VUINT8 time_out_of_out_over_temper;            /* End power when temperature is high for a certain time. */
}T_system_params;

extern const VUINT8 g_timing_Tbl[];
extern const T_system_params g_SysParams;        

#endif
