/** ###################################################################
**     Filename  : prx_lib.H
**     Project   : PowerReceiver
**     Processor : MC9RS08KB12CWJ
**     Component : 
**     Version   : Driver 01.00
**     Compiler  : CodeWarrior RS08 C Compiler
**     Date/Time : 11/22/2011, 5:54 PM
**     Abstract  :
**         This is header file of power receiver core module..
**         It contains constants and macro defines and function prototypes.
**     Settings  :
**     Contents  :
**         No public methods
**
** ###################################################################*/

#ifndef __Prx_lib_H
#define __Prx_lib_H

#include "defines.h"
#include "Cpu.h"
#include "IO_Map.h"

#define ADC_RESOLUTION          				10		/* This must match AdcLdd1_ADC_RESOLUTION parameter if using PEx generated files */

#define PK_TYPES 								23
#define MTIM_MOD_VLAUE 							100
#define PK_OUT_BUF_SIZE 						10

/*Set rectified power packet max period to 5sec, ~=60*(25+60)ms, packet tx period:25ms, max packet delay: 60ms*/
#define REC_PWR_TX_INTERVAL 60  

/*Test point*/
#define TEST_POINT_REC_VOL 						1
#define TEST_POINT_CTRL_ERR 					2
#define TEST_POINT_COM_DATA 					3

/*system error*/
#define SYS_ERR_NONE                          	0
#define SYS_ERR_OUT_DISCONNECT        			1
#define SYS_ERR_OUT_OVER_VOLTAGE    			2
#define SYS_ERR_OUT_UNDER_VOLTAGE  				3
#define SYS_ERR_OUT_OVER_CURRENT    			4
#define SYS_ERR_OUT_SHORT                 		5
#define SYS_ERR_IN_UNDER_VOLTAGE    			6
#define SYS_ERR_IN_OVER_VOLTAGE      			7
#define SYS_ERR_TEMP_OVER_HEAT       			8

/*Packet transmit status*/
#define COM_STS_PK_END      					0    /*Send packet end*/
#define COM_STS_PK_PREAMBLE 					1    /*Send packet preamble*/
#define COM_STS_PK_MESSAGE  					2    /*Send packet message*/
#define COM_STS_PK_DELAY  						3    /*Delay after packet sent*/

/*Byte transmit status*/
#define COM_STS_BYTE_BIT_START  				0    /*Byte send start bit*/
#define COM_STS_BYTE_BIT_BITS   				1    /*Byte send bit[0:7]*/
#define COM_STS_BYTE_BIT_PARITY 				2    /*Byte send parity bit*/
#define COM_STS_BYTE_BIT_STOP   				3    /*Byte send stop bit*/

/*Packet header*/
#define PK_HEADER_NULL               			0x00
#define PK_HEADER_SIGNAL_STRENGTH               0x01
#define PK_HEADER_END_POWER                     0x02
#define PK_HEADER_CONTROL_ERROR                 0x03
#define PK_HEADER_RECTIFIED_POWER               0x04
#define PK_HEADER_CHARGE_STATUS                 0x05
#define PK_HEADER_POWER_CONTROL_HOLDOFF         0x06
#define PK_HEADER_CONFIG                        0x51
#define PK_HEADER_IDENTIFY                      0x71
#define PK_HEADER_EXT_IDENTIFY                  0x81

/*Qi packet message data length*/
#define QI_PKDL_CFG 	 						5
#define QI_PKDL_IDE  							7
#define QI_PKDL_EIDE 							8

/*Max number of optional configuration packets */
#define MAX_OPT_CONF_PK_NUM     				7

#define ADC_BUF_SIZE 							8

/*Macro definitions*/
#ifdef KINETIS_L_DEVICE
	
/* For KL we setup the count value defined by microseconds */
#define TIME_250US 								250		//250u
#define TIME_1MS    							1000	//1000u

#else
	/* RS08 sets up clock divider differently for timer */
#define MTIM2_TIME_250US 						3
#define MTIM2_TIME_1MS    						5
#endif


#ifdef KINETIS_L_DEVICE

	/* don't think this is needed for now */

#else
/*GPIO list*/
typedef enum{
        /*Don't change the order! It's consistent with GPIO_Table */
        GPIO_COM,
        GPIO_DISCH_EN,
        GPIO_DC_DC_EN,
        GPIO_DC_DC_PG,
        GPIO_DBG1,
        GPIO_DBG2,
}GPIO_LIST;
#endif


/*power transfer phase*/
typedef enum
{
        QI_PHASE_SELECTION=0,
        QI_PHASE_PING,
        QI_PHASE_IDE_CFG,
        QI_PHASE_PWR_TRANS
}QI_POWER_TRANSFER_PHASE;

/*ide&cfg states*/
typedef enum{
        IDECFG_STATE_SEND_IDE_PK,
        IDECFG_STATE_SEND_EXT_IDE_PK,
        IDECFG_STATE_SEND_OPT_CFG_PK,
        IDECFG_STATE_SEND_CFG_PK
}IDECFG_STATE;

/*QI timing constrains list*/
typedef enum
{
        /*Don't change the order! It's consistent with g_QiTiming_Tbl*/

        /*Ping phase*/
        QI_TIMING_T_WAKE,
                                   
        /*ide&cfg phase*/
        QI_TIMING_T_SILENT,
                                   
        /*pwr_trans phase*/        
        QI_TIMING_T_DELAY_SHORT,
        QI_TIMING_T_DELAY_LONG,                                   
        QI_TIMING_T_CONTROL,

        /*power receiver timing*/        
}QITIMING_LIST;

/*end power code*/
typedef enum{
        EPC_UNKNOWN=0,
        EPC_CHARGE_COMPLETE,
        EPC_INTERNAL_FAULT,
        EPC_OVER_TEMPERATURE,
        EPC_OVER_VOLTAGE,
        EPC_OVER_CURRENT,
        EPC_BATTERY_FAILURE,
        EPC_RECONFIGURE,
        EPC_NO_RESPONSE
}END_POWER_CODE;

/*Qi communication timing point*/
typedef enum{
        TS_PK_END,                     /*0, time stamp during ping and ide&cfg*/
        
        /*Ping phase*/
                                   
        /*ide&cfg phase*/          
                                   
        /*pwr_trans phase*/        

        QI_TS_SIZE,                  /*10*/
}QI_TIME_STAMP_ID;

/*System error list*/
typedef enum{
        SYS_ERR_OUT_BUF_OVERFLOW,
        SYS_ERR_PK_SIZE_TOO_LARGE
}T_System_error_list;

/*packet transmission status*/
typedef enum{
        PK_TX_STOP,     //No packet in transmitting.
        PK_TX_DELAY,    //Packet transmitting end and delay 
        PK_TX_RUN,      //Packet in transmitting.
}T_packet_transmission_status;

/*Structure for packet send control*/
typedef struct
{
        VUINT16 parityBit;              /*parity bit of bytes in output buffer*/
        VUINT8 pkBuf[PK_OUT_BUF_SIZE];  /*packet buffer*/
        VUINT8 pkTxStatus;              /*Packet send .*/
        VUINT8 byteTxStatus;            /*0/1: buffer empty/full.*/

        /*preamble send flag*/
        VUINT8 preambleLeft;            /*preamble "1" left to send. */

        /*message send flag*/
        VUINT8 bitOffset;               /*Bit offset for byte send.*/
        VUINT8 byteOffset;              /*Byte offfset for packet send*/
        VUINT8 bytesLeft;               /*Bytes left when packet send*/
        VUINT8 setLow;                /*Whether set level low after packet sent.*/
        VUINT8 pkDelayCnt;      /*Delay t_silent time before next packet sent.*/
}T_packet_tx_control;


/*Power receiver control data*/
typedef struct
{
        /*State and phase control*/
        VUINT8 phase;                   /*Current power transfer phase*/
        //VUINT8 phaseLast;               /*Last time power transfer phase*/

        /*Ping control*/
        VUINT8 isPinged;                 /*Whether ping signal has been sent.*/
        

        /*ide&cfg control*/
        VUINT8 ideCfgState;            /*ide&cfg phase state. Receiving ide, ext-ide, opt-cfg or cfg.*/

        /*Power transfer phase*/
        VUINT8 pkDelay;
        VUINT8 ctrlErrCnt;
        VUINT8 isOutEnabled;                 /*Whether output enabled.*/
        VUINT8 DCon;                 /*Whether DC-DC on.*/

}T_system_control;

typedef struct
{
        VUINT16 vol;       /*voltage*/
        VUINT16 curr;      /*current*/
        VUINT16 temp;      /*temperature*/
}T_control_point;


/*Data extern*/
extern VUINT8 outBufSts;        /*Packet output buffer status. 0/1: empty or full.*/
extern VUINT8 bitClkFlag;       /*bit clock period. 0/1: first half period/second half period */


extern VUINT8 g_status;                             /*System running status*/
extern VUINT8 g_ErrorStatus;                        /*System error status*/
extern VUINT8 mtim_ms_cnt;                   /*millisecond count*/
//extern VUINT8 time_point_ms;                   /*millisecond timepoint*/
extern VUINT8 mtim_0_1_s_cnt;                /*0.1sec count*/
extern VUINT8 mtim_sec_cnt;
extern VUINT8 time_point_ms;
extern VUINT8 time_point_0_1_s;               /*0.1sec timepoint*/ 
extern VUINT8 t_wake_delay;

extern T_system_control g_systemCtrl;          /*system control data*/
extern T_packet_tx_control g_pkTxCtrl;         /*communication control data*/
extern T_control_point g_ctrlPoint_actual;
extern T_control_point g_ctrlPoint_desired;

extern VUINT8 sig_stren;                       /*signal strength*/
extern VUINT8 rec_vol;                         /*Vbus instant voltage*/
extern VUINT8 rec_vol_average;                 /*Vbus average voltage*/
extern VUINT8 out_vol;                         /*output voltage*/
extern VUINT8 out_cur;                         /*output current*/
extern VUINT8 sys_temp;                        /*system temperature*/
extern VINT8 rec_err;                          /*Vbus err*/
extern VINT8 out_err;                          /*DC-5v err*/
extern VUINT8 rec_vol_expected;                /*debug use*/
#ifdef POWER_GOOD_PROTECTION
extern VUINT8 dcdc_power_good;                      /*DC-DC power good signal*/
#endif
extern VUINT8 rec_pwr_percent;


extern VUINT8 end_power_code;           /*end power code*/
extern VUINT8 system_error;            /*system error status*/
extern VUINT16 system_runtime_status;           /*output status*/

extern VUINT8 rec_over_vol_cnt;
extern VUINT8 output_short_cnt;
extern VUINT8 output_disconnect_cnt;
extern VUINT8 output_over_curr_cnt;
extern VUINT8 output_over_vol_cnt;
extern VUINT8 output_under_vol_cnt;
//extern VUINT16 rec_under_voltage_cnt;
//extern VUINT16 rec_vol_stable_cnt;

#ifdef ADC_10_BITS
extern VUINT16 rec_vol_16b;
extern VUINT16 out_vol_16b;
extern VUINT16 out_cur_16b;
extern VUINT16 sys_temp_16b;

extern VINT16 rec_err_16b;
extern VINT16 out_err_16b;
#endif

#ifdef ADC_10_BITS_FOR_RX_POWER
extern VUINT16 out_vol_10b;
extern VUINT16 out_cur_10b;
//extern VUINT16 out_cur_10b_average;
//extern VUINT16 out_vol_average;
#endif

#ifndef KINETIS_L_DEVICE
#ifdef USE_PAGE_RAM_FOR_VARIABLES	//for RS08
#pragma DATA_SEG __PAGED_SEG PAGED_RAM
#endif
#endif //#ifndef KINETIS_L_DEVICE

#ifdef ADC_10_BITS_FOR_RX_POWER
extern VUINT16 out_cur_10b_average;
extern VUINT16 out_vol_average;
#endif


extern VUINT8 usb_charger_toggle;
extern VUINT8 rec_vol_buf[ADC_AVERAGE_WINDOW_SIZE_REC_VOL];    /*rectifier voltage buffer for ADC average*/
extern VUINT8 rec_vol_buf_idx;                                         /*buffer index to write*/

#ifdef _DBG
extern VUINT8 TestPoint;
extern VUINT8 logBufCnt;
extern VUINT8 logBuf[64];

/*Debug data*/
extern VUINT8 d_pkTxCnt;
extern VUINT8 d_byteTxCnt;
extern VUINT8 d_bitTxCnt;
extern VUINT8 d_bitClkCnt;

/*Bit0: 1: enable ping test.
*/
extern VUINT8 d_testFlag;

extern VUINT16 time_point_buf[QI_TS_SIZE];              /*Qi timing stamp record buffer*/
#endif

extern	long sdwReceivedPower;	// Signed in case a coefficient is negative
extern	VUINT32 dwOutputPower;
extern	VUINT32 sdwTemp;
extern	VUINT32 dwReceivedPower;
extern	VUINT32 dwVoltageData ;
extern	VUINT32 dwCurrentData ;

extern VUINT32 debug_dwVoltageData ;
extern VUINT32 debug_dwCurrentData ;
extern VUINT32 debug_dwOutputPower ;
extern VUINT32 debug_dwReceivedPower;

#ifndef KINETIS_L_DEVICE
#ifdef USE_PAGE_RAM_FOR_VARIABLES
#pragma DATA_SEG DEFAULT
#endif
#endif //#ifndef KINETIS_L_DEVICE

//------------------------------------------- Power Definitions --------------------------------------

// These definitions only account for the power dissipated by the load
#define QUIESCENT_POWER_PERCENT_OF_MAX_POWER	0		// (Temporary) 0%

// 5V output, up to 1A supplied current 
#define MAX_DEVICE_WATTAGE						5 // 5 Watts
#define MAX_POWER_MILLIWATTS					( MAX_DEVICE_WATTAGE * 1000 )

// Received Power and FOD definitions
#define MAX_POWER_MICROWATTS						( dword )( ( dword )5000 * ( dword )1000 )

#define MICROWATTS_PER_ADC_COUNT					( MAX_POWER_MICROWATTS / MAX_ADC_COUNTS )
#define MAX_POWER_MW								( ( dword )MAX_DEVICE_WATTAGE * 1000 )

#define MAX_ADC_COUNTS      					0x03FF
#ifdef KINETIS_L_DERIVATIVE
#define ADC_BITS								AdcLdd1_ADC_RESOLUTION	//Pull in from PEx component to make it automated 
#else
/* RS08 original */
#define ADC_BITS								10
#endif

// I_SENSE circuit definitions
#define R_CURRENT_SENSE_VALUE					0.02
#define I_SENSE_GAIN							50
#define OUTPUT_CURRENT_GAIN_MA_PER_VOLT 		( 1000.0 * ( 1.0 / ( R_CURRENT_SENSE_VALUE * I_SENSE_GAIN ) ) )
// VBAT_SENSE circuit definitions

/* Initial reference design may use different resistors although we need up to 8V range so this may not work */

#ifdef TOWER_PROTO_BOARD /*RS08 v1.2 original, also Kinetis initial debug*/
/* v1.2:  4.7k/2k;  v2.0: 47k/20k */
#define R108_VALUE								4700.0		//47000.0
#define R113_VALUE								2000.0		//20000.0
#else 
#define R108_VALUE								162000.0	/* 162K - R22 on wearables design*/
#define R113_VALUE								 47000.0	/* 47K - R24 on wearables design*/
#endif

#define VBAT_SENSE_GAIN 						( R113_VALUE / ( R113_VALUE + R108_VALUE ) )
#define OUTPUT_VOLTAGE_GAIN_MV_PER_VOLT 		( ( 1 / VBAT_SENSE_GAIN ) * 1000 )
#define MICROVOLTS_PER_ADC_COUNT				( word )( ( ADC_REFERENCE_VOLTAGE / MAX_ADC_COUNTS ) * 1000000 )

/**************************************************************************************************************/
/**************************************************************************************************************/
/**************************************************************************************************************/
/*					Define main MCU voltage here 															*/
/**************************************************************************************************************/
/**************************************************************************************************************/
/**************************************************************************************************************/
/* Initial reference design reference voltage */
#ifdef V1V8_SYSTEM	
/* Option for Kinetis */
#define ADC_REFERENCE_VOLTAGE					1.8
#else
/*RS08 and Kinetis*/
#define ADC_REFERENCE_VOLTAGE 					3.3
#endif
/**************************************************************************************************************/
/**************************************************************************************************************/
/**************************************************************************************************************/

#define ADC_COUNTS_PER_VOLT 						( word ) ( MAX_ADC_COUNTS / ADC_REFERENCE_VOLTAGE )
#define ADC_COUNTS_PER_MILLIVOLT					( ADC_COUNTS_PER_VOLT / 1000.0 )

/*********************************************************************************************/
/*********************************************************************************************/
/********************************************************************************
 * Added for Kinetis L porting
 * Using generic R1 and R2 values b/c resistor numbers are different between
 * hardware versions 
 *********************************************************************************/
/*********************************************************************************************/
/*********************************************************************************************/
#ifdef TOWER_PROTO_BOARD
/* Using v1.2A1 hardware */
#define VBUS_R1_VALUE							18000.0		/* 18K */
#define VBUS_R2_VALUE							 2000.0		/* 2K */
//#else
/* Using v2.0 hardware */
//#define VBUS_R1_VALUE							100000.0	/* 100K */
//#define VBUS_R2_VALUE							 20000.0	/* 20K */
#else
/* Wearables board */
#define VBUS_R1_VALUE							100000.0	/* 100K */	
#define VBUS_R2_VALUE							  9800.0	/* 9.8K */
#endif

	/*
	 * VBUS Parameters - Unfiltered voltage after rectifier (AC converted to DC)
	 * All 10-bit ADC values are right shifted by 2 to get 8-bit result.
	 */
#define VBUS_DIVIDER_FACTOR						( VBUS_R2_VALUE / ( VBUS_R1_VALUE + VBUS_R2_VALUE))	
#define VBUS_SENSE_GAIN						 	( 1 / VBUS_DIVIDER_FACTOR )		
#define VBUS_GAIN_MV_PER_VOLT 					( (1 / VBUS_SENSE_GAIN ) * 1000 )
#define VBUS_MICROVOLTS_PER_ADC_COUNT			( word )( (ADC_REFERENCE_VOLTAGE / MAX_ADC_COUNTS) * 1000000 )
#define VBUS_MILLIVOLTS_PER_ADC_COUNT			( word )( (ADC_REFERENCE_VOLTAGE / MAX_ADC_COUNTS) * 1000 ))
#define VBUS_MAX_VOLTAGE						( ADC_REFERENCE_VOLTAGE * VBUS_SENSE_GAIN )	//VBUS MAX
	/* VBUS ADC Values */
#define VBUS_ADC_COUNT_0V3						(VUINT16)((0.3 * (1<<ADC_RESOLUTION) ) / (VBUS_SENSE_GAIN * ADC_REFERENCE_VOLTAGE))
#define VBUS_ADC_COUNT_0V8						(VUINT16)((0.8 * (1<<ADC_RESOLUTION) ) / (VBUS_SENSE_GAIN * ADC_REFERENCE_VOLTAGE))
#define VBUS_ADC_COUNT_4V5						(VUINT16)((4.5 * (1<<ADC_RESOLUTION) ) / (VBUS_SENSE_GAIN * ADC_REFERENCE_VOLTAGE)) 
#define VBUS_ADC_COUNT_5V0						(VUINT16)((5.0 * (1<<ADC_RESOLUTION) ) / (VBUS_SENSE_GAIN * ADC_REFERENCE_VOLTAGE))
#define VBUS_ADC_COUNT_5V5						(VUINT16)((5.5 * (1<<ADC_RESOLUTION) ) / (VBUS_SENSE_GAIN * ADC_REFERENCE_VOLTAGE))
#define VBUS_ADC_COUNT_6V0						(VUINT16)((6.0 * (1<<ADC_RESOLUTION) ) / (VBUS_SENSE_GAIN * ADC_REFERENCE_VOLTAGE))
#define VBUS_ADC_COUNT_7V0						(VUINT16)((7.0 * (1<<ADC_RESOLUTION) ) / (VBUS_SENSE_GAIN * ADC_REFERENCE_VOLTAGE))
#define VBUS_ADC_COUNT_7V5						(VUINT16)((7.5 * (1<<ADC_RESOLUTION) ) / (VBUS_SENSE_GAIN * ADC_REFERENCE_VOLTAGE))
#define VBUS_ADC_COUNT_8V5						(VUINT16)((8.5 * (1<<ADC_RESOLUTION) ) / (VBUS_SENSE_GAIN * ADC_REFERENCE_VOLTAGE))
#define VBUS_ADC_COUNT_15V0						(VUINT16)((15.0 * (1<<ADC_RESOLUTION) ) / (VBUS_SENSE_GAIN * ADC_REFERENCE_VOLTAGE))
	/* VBUS ADC Values shifted from 10 to 8 bits - integer */
#define VBUS_ADC_INT_COUNT_0V3					((VUINT8)((VUINT16)(VBUS_ADC_COUNT_0V3) >> 2))
#define VBUS_ADC_INT_COUNT_0V8					((VUINT8)((VUINT16)(VBUS_ADC_COUNT_0V8) >> 2))
#define VBUS_ADC_INT_COUNT_4V5					((VUINT8)((VUINT16)(VBUS_ADC_COUNT_4V5) >> 2))
#define VBUS_ADC_INT_COUNT_5V0					((VUINT8)((VUINT16)(VBUS_ADC_COUNT_5V0) >> 2))
#define VBUS_ADC_INT_COUNT_5V5					((VUINT8)((VUINT16)(VBUS_ADC_COUNT_5V5) >> 2))
#define VBUS_ADC_INT_COUNT_6V0					((VUINT8)((VUINT16)(VBUS_ADC_COUNT_6V0) >> 2))
#define VBUS_ADC_INT_COUNT_7V0					((VUINT8)((VUINT16)(VBUS_ADC_COUNT_7V0) >> 2))
#define VBUS_ADC_INT_COUNT_7V5					((VUINT8)((VUINT16)(VBUS_ADC_COUNT_7V5) >> 2))
#define VBUS_ADC_INT_COUNT_8V5					((VUINT8)((VUINT16)(VBUS_ADC_COUNT_8V5) >> 2))
#define VBUS_ADC_INT_COUNT_15V0					((VUINT8)((VUINT16)(VBUS_ADC_COUNT_15V0) >> 2))

	/*
	 * VOUT Parameters - +5V Output voltage 
	 * All 10-bit ADC values are right shifted by 2 to get 8-bit result
	 */
#define VOUT_DIVIDER_FACTOR						VBAT_SENSE_GAIN
#define VOUT_SENSE_GAIN						 	( 1 / VOUT_DIVIDER_FACTOR )		
#define VOUT_GAIN_MV_PER_VOLT 					( (1 / VOUT_SENSE_GAIN ) * 1000 )
#define VOUT_MAX_VOLTAGE						( ADC_REFERENCE_VOLTAGE * VOUT_SENSE_GAIN )	//VOUT MAX
	/* VOUT ADC Values */
#define VOUT_ADC_COUNT_4V5						(VUINT16)((4.5 * (1<<ADC_RESOLUTION) ) / (VOUT_SENSE_GAIN * ADC_REFERENCE_VOLTAGE))
#define VOUT_ADC_COUNT_5V0						(VUINT16)((5.0 * (1<<ADC_RESOLUTION) ) / (VOUT_SENSE_GAIN * ADC_REFERENCE_VOLTAGE)) 
#define VOUT_ADC_COUNT_5V5						(VUINT16)((5.5 * (1<<ADC_RESOLUTION) ) / (VOUT_SENSE_GAIN * ADC_REFERENCE_VOLTAGE))
	/* VOUT ADC VAlues shifted from 10 to 8 bits - integer */
#define VOUT_ADC_INT_COUNT_4V5					((VUINT8)((VUINT16)(VOUT_ADC_COUNT_4V5) >> 2))
#define VOUT_ADC_INT_COUNT_5V0					((VUINT8)((VUINT16)(VOUT_ADC_COUNT_5V0) >> 2))
#define VOUT_ADC_INT_COUNT_5V5					((VUINT8)((VUINT16)(VOUT_ADC_COUNT_5V5) >> 2))
/* max output current - 1.5A Max - 10 bit result - VUINT16 in params.h */ 
#define VOUT_ADC_I_MAX_CURRENT_COUNT_1A5		(OUTPUT_CURRENT_GAIN_MA_PER_VOLT * (1.5 / ADC_REFERENCE_VOLTAGE))	//~450	
/* min output current - 10 MA Min - 8 bit result - VUINT8 in params.h */
#define VOUT_ADC_I_MIN_CURRENT_COUNT_50MA		((VUINT8)((VUINT16)(OUTPUT_CURRENT_GAIN_MA_PER_VOLT * (0.05 / ADC_REFERENCE_VOLTAGE))) >> 2)	
/*********************************************************************************************/
/*********************************************************************************************/
/********************************************************************************************************
 * End Kinetis L porting additions
 ********************************************************************************************************/
/*********************************************************************************************/
/*********************************************************************************************/

#define BATT_V_SENSE_MICROVOLTS_PER_ADC_COUNT		( dword )( MICROVOLTS_PER_ADC_COUNT / VBAT_SENSE_GAIN )
#define BATT_V_COUNTS_PER_MILLIVOLT 				( VBAT_SENSE_GAIN * ADC_COUNTS_PER_MILLIVOLT )

#define BATT_I_SENSE_MICROAMPS_PER_ADC_COUNT		( dword )( MICROVOLTS_PER_ADC_COUNT / ( I_SENSE_GAIN * R_CURRENT_SENSE_VALUE ) )
#define BATT_I_COUNTS_PER_MILLIAMP					( R_CURRENT_SENSE_VALUE* I_SENSE_GAIN * ADC_COUNTS_PER_MILLIVOLT )


//------------------ PowerLossDetection module defines -----------------
#define OUTPUT_POWER_16BIT_SHIFT_VALUE				0
#define OUTPUT_POWER_16BIT_MULTIPLIER				0
#define OUTPUT_POWER_PER_ADC_CNT_16BIT_SHIFT_VALUE	17
#define OUTPUT_POWER_PER_ADC_CNT_16BIT_MULTIPLIER	131072


// Calculate characteristics of the ADC
// NOTE: The 16-bit shift value is a constant used to insure the max counts squared is represented within a 16-bit variable.
//  For a 10-bit ADC, the max counts squared is 1046529.  Representing this as a 16-bit variable, we are required to shift right by '4'.
// NOTE: Cast ADC_COUNTS_SQUARED_MAX_VALUE definition to DWORD value to prevent compilers from calculating (0xFFF)^2 as negative number!
#define ADC_COUNTS_SQUARED_MAX_VALUE                ( dword )( 1.0 * MAX_ADC_COUNTS * MAX_ADC_COUNTS )
#define ADC_16BIT_SHIFT_VALUE                       ( word )( ( 2 * ADC_BITS ) - 16 )
#define ADC_16BIT_DIVISOR_VALUE                     ( word )( 1U << ADC_16BIT_SHIFT_VALUE )
#define ADC_COUNTS_SQUARED_16BIT_MAX_COUNTS         ( word )( ADC_COUNTS_SQUARED_MAX_VALUE / ADC_16BIT_DIVISOR_VALUE )

// Calculate the MAX ranges of ADC measurements supported by the hardware
#define OUTPUT_CURRENT_MAX_RANGE_MA_FLOAT           ( OUTPUT_CURRENT_GAIN_MA_PER_VOLT * ADC_REFERENCE_VOLTAGE )
#define OUTPUT_VOLTAGE_MAX_RANGE_MV_FLOAT           ( OUTPUT_VOLTAGE_GAIN_MV_PER_VOLT * ADC_REFERENCE_VOLTAGE )
                                                   // Results in mW with floating point decimal precision
#define OUTPUT_POWER_MAX_RANGE_FLOAT                ( ( OUTPUT_CURRENT_MAX_RANGE_MA_FLOAT * OUTPUT_VOLTAGE_MAX_RANGE_MV_FLOAT ) / 1000.0 )

// Calculate the terms used for Output Power assessment
#define POWER_PER_ADC_COUNTS_16BIT                  ( OUTPUT_POWER_MAX_RANGE_FLOAT / ADC_COUNTS_SQUARED_16BIT_MAX_COUNTS )
#define SCALED_POWER_PER_ADC_CNT_16BIT              ( dword )( ( POWER_PER_ADC_COUNTS_16BIT * OUTPUT_POWER_PER_ADC_CNT_16BIT_MULTIPLIER ) + 0.5 )



//---------------------------------- FOD Calculation definitions -------------------------------------
// The calculation to determine the Received power is (( Pout * Pout * 2ndOrderCoefficient ) + ( Pout * 1stOrderCoefficient ) + Offset)
// Where Pout is the output (load) power in mW. Therefore 2ndOrderCoefficient is in mW/mW^2 and 1stOrderCoefficient is in mW/mW

// Quadratic coefficient definition
// NOTE: The actual value of the second order coefficient is 1.174*10^-5
//		 Since floating point is not possible, multiply by 2^n where n causes
//		 the result to be as large as possible within a 16-bit limit.
#define QUADRATIC_COEFFICIENT_EXPONENT				32
#define QUADRATIC_COEFFICIENT						50423

// First order coefficient definition
// NOTE: The actual value of the first order coefficient is 1.0485
//		 Since floating point is not possible, multiply by 2^n where n causes
//		 the result to be as large as possible within a 16-bit limit.
#define LINEAR_COEFFICIENT_EXPONENT 				15
#define LINEAR_COEFFICIENT							34357

// Offset definition
#define OFFSET_MW									495

#define RECEIVED_POWER_SAMPLING_TIME				64 // 64ms (one sample per millisecond)
#define FOD_AVERAGE_COUNT_MAX						64 // 64 samples taken before calculating the average
#define FOD_AVERAGE_RIGHT_SHIFT 					6  // Right shift by 6 to devide by 64
#define FOD_PRECISION_SHIFT_AMOUNT					8  // Left shift by 8 to increase precision for PLD packet content calculation
// NOTE: This should be large enough to accomodate the PLD calculation time and packet building of less powerful micros
#define RECEIVED_POWER_CALCULATION_TIME 			5  // 5 ms, this is the maximum expected time for less powerful micros
// After calculating the output power in ADC counts, shift by this amount to remove the extra precision.
// This number should bethe same as the number of bits the ADC result is in.
#define OUTPUT_POWER_FOD_CALCULATOIN_SHIFT			10

//-----------
// FUNCTIONS
//-----------
extern void rx_enable(void);
extern void rx_disable(void);
extern VUINT8 detect_adapter(void);
extern void usb_enable(void);
extern void usb_disable(void);
extern void init_system_variables(void);
extern VUINT32 FODCalculateReceiverPower(void);
//extern void SET_MTIM2_TIME(VUINT8 length);			//RS08 original
extern void SET_TIMER1_INTERRUPT_TIME(VUINT32 time_in_us);	//KL update


/*Function prototype*/
extern void init_HW(void);
extern void init_com(void);

extern VUINT8 system_monitor(void);
extern void ADC_average(void);
extern void read_current_sense_offset();
extern VUINT8 system_check(void);
extern void wait_by_nop(VUINT8 count);
extern void qi_charging_state_machine(void);
extern VUINT8 selection_phase(VUINT8 *phase);
extern VUINT8 ping_phase(VUINT8 *phase);
extern VUINT8 ide_cfg_phase(VUINT8 *phase);
extern VUINT8 power_transfer_phase(VUINT8 *phase);
//VUINT8 is_adc_averaged(void);
void vol2ss(void);
void vol2err(void);
#ifdef ADC_10_BITS
VUINT8 power_down_detect_16b(void);
void vol2ss_16b(void);
void vol2err_16b(void);
#endif
void CalcRectifiedPwrPercent(void);
VUINT8 charge_status(void);
VUINT8 GetPKSize(VUINT8 header);
#ifdef USE_PAGE_RAM_FOR_VARIABLES
void prepare_packet_send(void);
#else
void prepare_packet_send(void);
#endif
VUINT8 calc_parity(VUINT8 Byte);
VUINT8 getPkTxSts(void);
void packet_send(void);
extern VUINT8 bitIO_read(void);
extern void bitIO_clear(void);
extern void bitIO_invert(void);
extern VUINT8 bit_send(VUINT8 Bit);
void setSysErr(VUINT8 errBit);
extern void invert_dbg_gpio(VUINT8 io);
extern void set_dbg_gpio(VUINT8 io);
extern void clr_dbg_gpio(VUINT8 io);
extern void set_GPIO(VUINT8 ID);
extern void clear_GPIO(VUINT8 ID);
extern void invert_GPIO(VUINT8 ID);
extern VUINT8*  get_GPIO_addr(VUINT8 ID, VUINT8* mask);
extern void output_enable(void);
extern void output_disable(void);
extern VUINT8 ADC_read(VUINT8 u8Channel);
extern VUINT16 ADC_read_16b(VUINT8 u8Channel);
extern VUINT8 get_time(void);

#if 0
bool timing_check(VUINT8 timId, VUINT8 TSid);
void save_time_point(VUINT8 TP);
VUINT16 get_time_point(VUINT8 TP);
VUINT16 CalcTimeLen(VUINT8 TP);
void delay_loop(VUINT16 delay);
#endif

#ifdef _DBG
#ifdef USE_PAGE_RAM_FOR_VARIABLES
void data_log(VUINT8 *__far data, VUINT8 TP, VUINT8 size);
#else
void data_log(VUINT8 *data, VUINT8 TP, VUINT8 size);
#endif
#endif

#endif  //#ifndef __Prx_lib_H

