/*
 * spi.c
 *
 *  Created on: Nov 24, 2014
 */

#include "Cpu.h"
#include "Events.h"
#include "UART0.h"
#include "PTE.h"
#include "PTC.h"
#include "PTD.h"
#include "SM1.h"
#include "PTB.h"
/* Including shared modules, which are used for whole project */
#include "PE_Types.h"
#include "PE_Error.h"
#include "PE_Const.h"
#include "IO_Map.h"
#include "common/common.h"

#include "spi.h"

int spiWaitRxNotEmpty()
{
	unsigned int timeout = 1000000;
	while(--timeout){
		if(SPI1_S & SPI_S_SPRF_MASK)
			return 0;
	}
	return -1;
}

char MultiByteWriteSPI(unsigned char u8RegisterAddress, const unsigned char *u8Buffer, char numBytes)
{
	short int i;
	//char ch;
		
	// Set CS# low
	GPIOD_PDOR &= ~0x10 ;
	//Send register address
	
	spiWaitTxEmpty();
	
	SPI1_DL = u8RegisterAddress | 0x80;
	
	//printf(".");
	
	if(spiWaitRxNotEmpty() == -1)
		return -1;
	SPI1_DL;		//dummy read
	
	for(i=0; i<numBytes; i++)
	{
		// Send data
		spiWaitTxEmpty();
		SPI1_DL = u8Buffer[i];
		if(spiWaitRxNotEmpty() == -1) 
			return -2;
		SPI1_DL;
	}
	//spiWaitTxEmpty();
	// Set CS# high
	GPIOD_PDOR |= 0x10 ;

	return 0;
}

char MultiByteReadSPI(unsigned char u8RegisterAddress, unsigned char *u8Buffer, char numBytes)
{
	
	unsigned int i;

	// Set CS# low
	GPIOD_PDOR &= ~0x10;
	
	// Send register address to read from
	spiWaitTxEmpty();
	SPI1_DL = u8RegisterAddress;
	
	//////////////////// REVIEW DUMMY READ MAY POSSIBLY BE FOR MMA ONLY ///////////////////////// 
	
	// Dummy read
	if(spiWaitRxNotEmpty() == -1) return -1;
	i = SPI1_DL;
	
	// Added dummy read for MMA, need to check with another device
	//spiWaitTxEmpty(); //remove?
	//SPI1_DL = 0x00;

	// get data
	//while(!(SPI1_S & SPI_S_SPRF_MASK)){} //remove?
	//u8Buffer[i] = SPI1_DL;
	
	for(i=0; i<numBytes; i++)
	{
		// Toggle CLK
		spiWaitTxEmpty();
		SPI1_DL = 0x00;
		
		// get data
		if(spiWaitRxNotEmpty() == -1) return -1;
		u8Buffer[i] = SPI1_DL;
	}

	// Set CS# high
	GPIOD_PDOR |= 0x10 ;

	return 0;
}
