/*
 * File:        uart.c
 * Purpose:     Provide common UART routines for serial IO
 *
 * Notes:       
 *              
 */


/* Including needed modules to compile this module/procedure */
#include "Cpu.h"
#include "Events.h"
#include "UART0.h"
/* Including shared modules, which are used for whole project */
#include "PE_Types.h"
#include "PE_Error.h"
#include "PE_Const.h"
#include "IO_Map.h"

#include "WaRP_DB.h"
#include "common/common.h"
#include "uart.h"

/********************************************************************/
/*
 * Wait for a character to be received on the specified UART
 *
 * Parameters:
 *  channel      UART channel to read from
 *
 * Return Values:
 *  the received character
 */
char uart_getchar (UART_MemMapPtr channel)
{
    /* Wait until character has been received */
    while (!(UART_S1_REG(channel) & UART_S1_RDRF_MASK));
    
    /* Return the 8-bit data from the receiver */
    return UART_D_REG(channel);
}
/********************************************************************/
/*
 * Wait for space in the UART Tx FIFO and then send a character
 *
 * Parameters:
 *  channel      UART channel to send to
 *  ch			 character to send
 */ 
void uart_putchar (UART_MemMapPtr channel, char ch)
{
	/* Wait until space is available in the FIFO */
    while(!(UART_S1_REG(channel) & UART_S1_TDRE_MASK));
    
    /* Send the character */
    UART_D_REG(channel) = (uint8)ch;
 }

void uart_irq_putchar (UART_MemMapPtr channel, char ch)
{	
	char temp;
	static unsigned long int j = 0;
	while(!(UartTxQueue.count < UART_QUEUE_SIZE)){
		j++;
	}
	disableUart0TxInt();
	temp = uartQueuePut(&UartTxQueue, ch);
	enableUart0TxInt();
	if(temp){
		PE_DEBUGHALT();
	}
	
	
//	// If UART TX Reg is ready and UART TX buffer queue is empty
//	if(uartTxRegEmpty() && UartTxQueue.count == 0){
//		uartQueuePut(&UartTxQueue, ch);
//		// Send the character
//		//UART_D_REG(channel) = (uint8)ch;
//		
//		// Enable UART TX interrupt
//		UART0_C2 |= UART0_C2_TIE_MASK;
//	}else if(!uartTxRegEmpty()){
//		// If UART TX is busy and UART buffer is full, wait for space in buffer
//		while(!(UartTxQueue.count < UART_QUEUE_SIZE));
//		uartQueuePut(&UartTxQueue, ch);
//		UART0_C2 |= UART0_C2_TIE_MASK;
//		//if(uartTxRegEmpty() && UartTxQueue.count != 0){
//		//	uartQueueGet(&UartTxQueue, &ch);
//		//	UART_D_REG(channel) = ch;
//		}
//	}else 
//	if(uartTxRegEmpty() && UartTxQueue.count != 0){
//		// If UART TX is ready and UART buffer queue has data, then something bad has occurred.
//		PE_DEBUGHALT();	
//	}
 }


/********************************************************************/
/*
 * Check to see if a character has been received
 *
 * Parameters:
 *  channel      UART channel to check for a character
 *
 * Return values:
 *  0       No character received
 *  1       Character has been received
 */
int uart_getchar_present (UART_MemMapPtr channel)
{
    return (UART_S1_REG(channel) & UART_S1_RDRF_MASK);
}
/********************************************************************/
    
